// $Id: OopsCollectionFilterRule.m,v 1.1.1.1 2000/03/02 10:39:40 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsCollectionFilterRule.h"

@interface OopsCollectionFilterRule (PrivateComplianceTesting)

- (BOOL)_performAndComplianceTestWithMessage:(Message *)message;
- (BOOL)_performOrComplianceTestWithMessage:(Message *)message;
- (BOOL)_performNotComplianceTestWithMessage:(Message *)message;

@end

@implementation OopsCollectionFilterRule

- (void)setRules:(NSArray *)someRules {
	[someRules retain];
	[rules release];
	rules = someRules;
}

- (NSArray *)rules {
	return rules;
}

- (BOOL)compliedToByMessage:(Message *)message {
	NSString *myType = nil;

	if (!message) {
		return NO;
	}

	myType = [self type];

	// Default rule operator is 'and', hence the check for nil.
    if (!myType || [myType caseInsensitiveCompare:@"and"] == NSOrderedSame) {
		return [self _performAndComplianceTestWithMessage:message];

    } else if ([myType caseInsensitiveCompare:@"or"] == NSOrderedSame) {
        return [self _performOrComplianceTestWithMessage:message];

    } else if ([myType caseInsensitiveCompare:@"not"] == NSOrderedSame) {
        return [self _performNotComplianceTestWithMessage:message];
    }

	NSLog(@"%@: unsupported rule type %@.", self, myType);
	return NO;
}

- (BOOL)loadFromPropertyListRepresentation:(NSDictionary *)dict {
    NSArray *array = nil;

    if (![super loadFromPropertyListRepresentation:dict]) {
        return NO;
    }

    array = [dict objectForKey:OopsCollectionFilterRuleRulesKey];
    if ([array isKindOfClass:[NSArray class]]) {
		NSMutableArray *rulesArray = [[NSMutableArray alloc] initWithCapacity:[array count]];
		int i, count = [array count];

		for (i=0; i<count; i++) {
			NSDictionary *dict = [array objectAtIndex:i];
			OopsFilterRule *aRule = nil;

            aRule = [[self class] ruleFromPropertyListRepresentation:dict];
			if (aRule) {
				[rulesArray addObject:aRule];
			}
		}

        [self setRules:rulesArray];
		[rulesArray release];
    }

    return YES;
}

- (NSMutableDictionary *)propertyListRepresentation {
    NSMutableDictionary *dict = nil;
    id obj = nil;

	dict = [super propertyListRepresentation];

    if (obj = [self rules]) {
        [dict setObject:obj forKey:OopsCollectionFilterRuleRulesKey];
    }

    return dict;
}

@end

@implementation OopsCollectionFilterRule (PrivateComplianceTesting)

- (BOOL)_performAndComplianceTestWithMessage:(Message *)message {
	NSArray *myRules = [self rules];
	NSEnumerator *ruleEnum = [myRules objectEnumerator];
    OopsFilterRule *aRule = nil;

    NSLog(@"%@: Performing AND search: %@", self, [self propertyListRepresentation]);

	// A message can't comply if we contain no rules.
    if (![myRules count]) return NO;

	while (aRule = [ruleEnum nextObject]) {
        if (![aRule compliedToByMessage:message]) {
			// Oops, the message didn't comply to the rule.
			return NO;
		}
	}

	// Message complied to all rules.
    return YES;
}

- (BOOL)_performOrComplianceTestWithMessage:(Message *)message {
    NSArray *myRules = [self rules];
    NSEnumerator *ruleEnum = [myRules objectEnumerator];
    OopsFilterRule *aRule = nil;

    NSLog(@"%@: Performing OR search: %@", self, [self propertyListRepresentation]);

    // A message can't comply if we contain no rules.
    if (![myRules count]) return NO;

    while (aRule = [ruleEnum nextObject]) {
        if ([aRule compliedToByMessage:message]) {
            // Message complied to the rule. Return directly.
            return YES;
        }
    }

    // Message didn't comply ay all.
    return NO;
}

- (BOOL)_performNotComplianceTestWithMessage:(Message *)message {
    NSArray *myRules = [self rules];
    NSEnumerator *ruleEnum = [myRules objectEnumerator];
    OopsFilterRule *aRule = nil;

    NSLog(@"%@: Performing NOT search: %@", self, [self propertyListRepresentation]);

    // A message can't comply if we contain no rules.
    if (![myRules count]) return NO;

    while (aRule = [ruleEnum nextObject]) {
        if ([aRule compliedToByMessage:message]) {
            // Oops, the message DID comply to the rule. Return immediately.
            return NO;
        }
    }

    // Message didn't comply at all. Good for us.
    return YES;
}

@end

NSString *OopsCollectionFilterRuleRulesKey = @"rules";

/* Revision history:
 * $Log: OopsCollectionFilterRule.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:40  malte
 * Initial release
 *
 */
