// $Id: OopsFilterRule.m,v 1.1.1.1 2000/03/02 10:39:40 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsFilterRule.h"

static NSMutableDictionary *_registeredClasses = nil;

@implementation OopsFilterRule

+ (void)registerClass:(Class)ruleClass forType:(NSString *)typeName {
    if (ruleClass == Nil || typeName == nil || [typeName isEqualToString:@""]) {
        return;
    }

    if (_registeredClasses == nil) {
        _registeredClasses = [[NSMutableDictionary alloc] init];
    }

    [_registeredClasses setObject:ruleClass forKey:typeName];
}

+ (Class)classForType:(NSString *)typeName {
    Class aClass = [_registeredClasses objectForKey:typeName];

    if (!aClass) {
        aClass = [_registeredClasses objectForKey:@"default"];
        // NSLog(@"%@: no registered rule for key %@. Will use \"default\" instead.", self, typeName, aClass);
    }

    if (!aClass) {
        if ([_registeredClasses count]) {
            aClass = [[_registeredClasses allValues] objectAtIndex:0];
            // NSLog(@"%@: no default rule registered. Will use %@ instead.", self, aClass);
        } else {
            // NSLog(@"%@: no registered rules.", self);
        }
    }

    return (aClass) ? aClass : Nil;
}

+ (OopsFilterRule *)ruleFromPropertyListRepresentation:(NSDictionary *)dict {
    NSString *ruleType = nil;
    Class ruleClass = Nil;
    OopsFilterRule *rule = nil;

	if (!dict || ![dict isKindOfClass:[NSDictionary class]]) {
		NSLog(@"%@: No property list or property list of wrong class %@", self, dict);
		return nil;
	}

    // Get the class of the rule.
    ruleType = [dict objectForKey:OopsFilterRuleTypeKey];
    ruleClass = [self classForType:ruleType];

    if (ruleClass == Nil) {
        NSLog(@"%@: unable to create rule. No class registered to handle type '%@'.", self, ruleType);
        return nil;
    }

    rule = [[ruleClass alloc] init];
    if (![rule loadFromPropertyListRepresentation:dict]) {
        NSLog(@"%@: unable to create rule. Error loading from property list %@.", self, dict);
        [rule release];
        return nil;
    }

    return rule;
}

- (void)setType:(NSString *)aType {
    [aType retain];
    [type release];
    type = aType;
}

- (NSString *)type {
    return type;
}

- (BOOL)compliedToByMessage:(Message *)message {
    return NO;
}

- (BOOL)loadFromPropertyListRepresentation:(NSDictionary *)dict {
    [self setType:[dict objectForKey:OopsFilterRuleTypeKey]];
    return YES;
}

- (NSMutableDictionary *)propertyListRepresentation {
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithCapacity:8];
    id obj = nil;

    if (obj = [self type]) {
        [dict setObject:obj forKey:OopsFilterRuleTypeKey];
    }

    return [dict autorelease];
}

@end

NSString *OopsFilterRuleTypeKey = @"type";

/* Revision history:
 * $Log: OopsFilterRule.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:40  malte
 * Initial release
 *
 */
