// $Id: OopsMessageFilter.m,v 1.1.1.1 2000/03/02 10:39:40 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsMessageFilter.h"
#import "OopsFilterRule.h"
#import "Message.h"

@implementation OopsMessageFilter

- (id)init {
	if (self = [super init]) {
		name = nil;
		order = 0;
		destinationStorePath = nil;
		rule = nil;
	}
	return self;
}

- (void)dealloc {
	[name release];
	[destinationStorePath release];
	[rule release];
	[super dealloc];
}

- (void)setName:(NSString *)aName {
	[aName retain];
	[name release];
	name = aName;
}

- (NSString *)name {
	return name;
}

- (void)setOrder:(int)anOrder {
	order = anOrder;
}

- (int)order {
	return order;
}

- (void)setDestinationStorePath:(NSString *)aPath {
	[aPath retain];
	[destinationStorePath release];
	destinationStorePath = aPath;
}

- (NSString *)destinationStorePath {
    return destinationStorePath;
}

- (void)setRule:(OopsFilterRule *)aRule {
	[aRule retain];
	[rule release];
	rule = aRule;
}

- (OopsFilterRule *)rule {
	return rule;
}

- (BOOL)compliedToByMessage:(Message *)message {
	if (!message || !rule) {
		NSLog(@"%@: message or rule not present.", self);
		return NO;
	}

	// NSLog(@"\n%@: Testing compliance of message...", self);
	return [[self rule] compliedToByMessage:message];
}

- (BOOL)loadFromPropertyListRepresentation:(NSDictionary *)dict {
	id obj = nil;
	NSString *nameValue = nil;
	int orderValue = 0;
	NSString *pathValue = nil;
	OopsFilterRule *ruleValue = nil;

    obj = [dict objectForKey:OopsMessageFilterNameKey];
    if (!obj || ![obj isKindOfClass:[NSString class]]) {
        NSLog(@"%@: unable to load from property list. %@", self, (!obj) ? @"Key 'name' not present." : @"Object for key 'name' of wrong class.");
		return NO;
	}
	nameValue = obj;

	obj = [dict objectForKey:OopsMessageFilterOrderKey];
	if (!obj || (![obj isKindOfClass:[NSNumber class]] && ![obj isKindOfClass:[NSString class]])) {
		NSLog(@"%@: unable to load from property list. %@", self, (!obj) ? @"Key 'order' not present" : @"Object for key 'order' of wrong class.");
        return NO;
	}
	#warning Todo: look out for exceptions.
	orderValue = [(NSNumber *)obj intValue];

	obj = [dict objectForKey:OopsMessageFilterDestinationKey];
	if (!obj || ![obj isKindOfClass:[NSString class]]) {
        NSLog(@"%@: unable to load from property list. %@", self, (!obj) ? @"Key 'destination' not present." : @"Object for key 'destination' of wrong class.");
        return NO;
    }
    pathValue = obj;

    obj = [dict objectForKey:OopsMessageFilterRuleKey];
	if (!obj) {
        NSLog(@"%@: unable to load from property list. Key 'rule' not present.", self);
        return NO;
	}

    obj = [OopsFilterRule ruleFromPropertyListRepresentation:obj];
	if (!obj) {
        NSLog(@"%@: unable to load from property list. Object for key 'rule' of bad format.", self);
        return NO;
	}
	ruleValue = obj;

	[self setName:nameValue];
	[self setOrder:orderValue];
	[self setDestinationStorePath:pathValue];
	[self setRule:ruleValue];
	return YES;
}

- (NSDictionary *)propertyListRepresentation {
	NSMutableDictionary *dict = [[NSMutableDictionary alloc] initWithCapacity:4];
	id obj = nil;

	if (obj = [self name]) {
        [dict setObject:obj forKey:OopsMessageFilterNameKey];
	}

	if (obj = [NSNumber numberWithInt:[self order]]) {
        [dict setObject:obj forKey:OopsMessageFilterOrderKey];
	}

	if (obj = [self destinationStorePath]) {
        [dict setObject:obj forKey:OopsMessageFilterDestinationKey];
	}

	if (obj = [[self rule] propertyListRepresentation]) {
        [dict setObject:obj forKey:OopsMessageFilterRuleKey];
	}

	return [dict autorelease];
}

@end

NSString *OopsMessageFilterNameKey = @"name";
NSString *OopsMessageFilterOrderKey = @"order";
NSString *OopsMessageFilterDestinationKey = @"destination";
NSString *OopsMessageFilterRuleKey = @"rule";

/* Revision history:
 * $Log: OopsMessageFilter.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:40  malte
 * Initial release
 *
 */
