// $Id: OopsMessageRouter.m,v 1.1.1.1 2000/03/02 10:39:38 malte Exp $
//
// This file is part of the OopsMailFilter Bundle for Apple's MailViewer application.
// Copyright (c) 1999-2000 Oops AB. All rights reserved.
//
// The source file window should be wide enough to fit this paragraph. When using ProjectBuilder by
// Apple, switch on the option "Indent wrapped lines" and set the indentation to 4 spaces.
//
// See end of file for revision history.
//

#import "OopsMessageRouter.h"
#import "MessageStore.h"
#import "LocalAccount.h"
#import "OopsMessageFilter.h"
#import "OopsMessageFilterList.h"

#import "OopsFilterRule.h"
#import "OopsSimpleFilterRule.h"
#import "OopsCollectionFilterRule.h"

@implementation OopsMessageRouter

+ (void)initialize {
	if (self != [OopsMessageRouter class]) {
		return;
	}

    [MessageRouter registerRouterClass:self];

	// Register filter rules.
    [OopsFilterRule registerClass:[OopsSimpleFilterRule class] forType:@"default"];
	[OopsFilterRule registerClass:[OopsSimpleFilterRule class] forType:@"simple"];
    [OopsFilterRule registerClass:[OopsSimpleFilterRule class] forType:@"header"];
    [OopsFilterRule registerClass:[OopsSimpleFilterRule class] forType:@"body"];

    [OopsFilterRule registerClass:[OopsCollectionFilterRule class] forType:@"and"];
    [OopsFilterRule registerClass:[OopsCollectionFilterRule class] forType:@"or"];
    [OopsFilterRule registerClass:[OopsCollectionFilterRule class] forType:@"not"];
}

- (id)init {
	if (self = [super init]) {
	}
	return self;
}

- (void)dealloc {
	[super dealloc];
}

+ (float)willingnessToRouteForStore:fp12 {
	/* Perhaps we should be more conservative to what we are willing to route. */

    // NSLog(@"%@: +willingnessToRouteForStore: %@", NSStringFromClass(self), fp12);
	// Route them all!
	return 1.0;
}

- (MessageStore *)storeForMessageFilter:(OopsMessageFilter *)filter {
    NSString *path = nil;
    MessageStore *store = nil;

    path = [filter destinationStorePath];
    if (!path) {
        NSLog(@"%@: filter %@ has no path", self, filter);
        return nil;
    }

    if (![path isAbsolutePath]) {
        LocalAccount *account = [LocalAccount localAccount];

		if (!account) {
			NSLog(@"%@: unable to determine destination store path for filter %@. Destination path %@ is not absolute and there is no local account present.", self, filter, path);
			return nil; 
		}

		path = [[account path] stringByAppendingPathComponent:path];
	}

    /* Is this step really needed?!? */
    // Is there an open store?
    store = [MessageStore currentlyOpenStoreForPath:path];
    if (store) {
        return store;
    }

    store = [MessageStore storeForPath:path];
    if (store) {
        return store;
    }

    store = [MessageStore storeForPath:path create:YES readOnly:NO];
    if (store) {
        NSLog(@"%@: created store for path %@", self, path);
        return store;
    }

    NSLog(@"%@: unable to create store for path %@", self, path);
    return nil;
}

+ (MessageStore *)localAccountPrimaryMessageStore {
    LocalAccount *account = nil;
    MessageStore *store = nil;

    account = [LocalAccount localAccount];
    if (!account) {
        NSLog(@"%@: no local account", NSStringFromClass(self));
        return nil;
    }

    store = [account primaryMessageStore];
    if (!store) {
        NSLog(@"%@: local account has no primary message store", NSStringFromClass(self));
        return nil;
    }

    return store;
}

- destinationStoreForMessage:message {
	MessageStore *store = nil;
	OopsMessageFilterList *list = [OopsMessageFilterList filterList];
	NSEnumerator *filterEnum = [list filterEnumerator];
	OopsMessageFilter *filter = nil;
	BOOL foundMatchingFilter = NO;

	while (filter = [filterEnum nextObject]) {
        if ([filter compliedToByMessage:message]) {
            foundMatchingFilter = YES;
            store = [self storeForMessageFilter:filter];
			break;
		}
	}

    if (!foundMatchingFilter) {
        // NSLog(@"%@: couldn't find filter for message <%@: 0x%06x>", self, NSStringFromClass([message class]), message);
		// Fall through.
	}

	if (!store) {
		// NSLog(@"%@: couldn't find message store. Falling back on default behaviour.", self);
        store = [super destinationStoreForMessage:message];
	}

	return store;
}

- (BOOL)routeMessagesSeparately {
	return YES;
}

@end

/* Revision history:
 * $Log: OopsMessageRouter.m,v $
 * Revision 1.1.1.1  2000/03/02 10:39:38  malte
 * Initial release
 *
 */
